package authentication;

import java.security.MessageDigest;

/**
 * Tda pro administraci uivatel
 */
public class UserManager 
{
/** Spojen s databz */
private java.sql.Connection connection;
/** Pihlaovac jmno do DB. */
private java.lang.String connectionName;
/** Pihlaovac heslo do DB */
private java.lang.String connectionPassword;
/** URL pro pipojen k DB.*/
private java.lang.String connectionURL;
/** ifrovac algoritmus pro ukldn hesel */
private java.lang.String digest;
/** Jmno JDBC ovladae pro prci s databz. */
private java.lang.String driverName;
/** Nzev sloupce se jmny rol */
private java.lang.String roleNameCol;
/** Jmno sloupce s pstupovmi hesly. */
private java.lang.String userCredCol;
/** Jmno sloupce s jmny uivatel. */
private java.lang.String userNameCol;
/** Jmno tabulky uivatel.*/
private java.lang.String userTable;
/** Jmno tabulky rol. */
private java.lang.String userRoleTable;

/**
 * Vytvo instanci
 */
public UserManager()
{
	super();
}
/**
 * Uzave spojen s DB.
 * @exception java.sql.SQLException - problm s databz.
 */
public void closeConnection() throws java.sql.SQLException 
{
	java.sql.Connection c = getConnection();
	if (c != null)
	{
		c.close();
		setConnection(null);
	}
}
/**
 * Metoda <code>convert</code> i s dokumentac je pevzata ze zdrojovch text
 * produktu TomCat (soubor HexUtil.java).<br/>
 * Convert a byte array into a printable format containing a
 * String of hexadecimal digit characters (two per byte).
 *
 * @param bytes Byte array representation
 */
public static String convert(byte bytes[]) 
{
 StringBuffer sb = new StringBuffer(bytes.length * 2);
 for (int i = 0; i < bytes.length; i++) 
 {
  sb.append(convertDigit((int) (bytes[i] >> 4)));
  sb.append(convertDigit((int) (bytes[i] & 0x0f)));
 }
 return (sb.toString());
}
/**
 * Metoda <code>convertDigit</code> i s dokumentac je pevzata ze zdrojovch text
 * produktu TomCat (soubor HexUtil.java).<br/>
 * [Private] Convert the specified value (0 .. 15) to the corresponding
 * hexadecimal digit.
 *
 * @param value Value to be converted
*/
private static char convertDigit(int value) 
{
 value &= 0x0f;
 if (value >= 10)
    return ((char) (value - 10 + 'a'));
 else
    return ((char) (value + '0'));
}
/**
 * Metoda <code>Digest</code> i s dokumentac je pevzata ze zdrojovch text
 * produktu TomCat (soubor RealmBase.java).<br/> 
 * Digest password using the algorithm especificied and
 * convert the result to a corresponding hex string.
 * If exception, the plain credentials string is returned
 *
 * @param credentials Password or other credentials to use in
 *  authenticating this username
 * @param algorithm Algorithm used to do th digest
*/
public String Digest(String credentials, String algorithm) 
{
 try 
 { // Obtain a new message digest with "digest" encryption
   MessageDigest md =
   (MessageDigest) MessageDigest.getInstance(algorithm).clone();
   // encode the credentials
   md.update(credentials.getBytes());
   // Digest the credentials and return as hexadecimal
   return (convert(md.digest()));
 } 
 catch(Exception ex) 
 {
   ex.printStackTrace();
   return credentials;
 }
}
/**
 * Je-li instance likvidovna, uzave spojen.
 * @exception java.sql.SQLException - problm s databz.
 */
public void finalize() throws java.sql.SQLException 
{
	closeConnection();
}
/**
 * Vrt spojen s databz
 * @return java.sql.Connection je spojen s databz.
 */
public java.sql.Connection getConnection() 
{
	return connection;
}
/**
 * Vrt pihlaovac jmno do DB.
 * @return java.lang.String je pihlaovac jmno do DB.
 */
public java.lang.String getConnectionName() 
{
	return connectionName;
}
/**
 * Nastav pihlaovac heslo do DB.
 * @return java.lang.String je pihlaovac heslo do DB.
 */
public java.lang.String getConnectionPassword() 
{
	return connectionPassword;
}
/**
 * Vrt URL pro pipojen k DB.
 * @return java.lang.String je URL pro pipojen k DB.
 */
public java.lang.String getConnectionURL() 
{
	return connectionURL;
}
/**
 * Vrt ifrovac algoritmus pro ukldn hesel.
 * @return java.lang.String je ifrovac algoritmus pro ukldn hesel.
 */
public java.lang.String getDigest() 
{
	return digest;
}
/**
 * Vrt jmno JDBC ovladae pro prci s databz.
 * @return java.lang.String je jmno JDBC ovladae pro prci s databz.
 */
public java.lang.String getDriverName() 
{
	return driverName;
}
/**
 * Vrt nzev sloupce se jmny rol.
 * @return java.lang.String je jzev sloupce se jmny rol.
 */
public java.lang.String getRoleNameCol() 
{
	return roleNameCol;
}
/**
 * Vrt jmno sloupce s pstupovmi hesly.
 * @return java.lang.String je jmno sloupce s pstupovmi hesly.
 */
public java.lang.String getUserCredCol() 
{
	return userCredCol;
}
/**
 * Vrt jmno sloupce s jmny uivatel.
 * @return java.lang.String je jmno sloupce s jmny uivatel.
 */
public java.lang.String getUserNameCol() 
{
	return userNameCol;
}
/**
 * Vrt jmno tabulky rol.
 * @return java.lang.String je jmno tabulky rol.
 */
public java.lang.String getUserRoleTable() 
{
	return userRoleTable;
}
/**
 * Vrt jmno tabulky uivatel.
 * @return java.lang.String je jmno tabulky uivatel.
 */
public java.lang.String getUserTable() 
{
	return userTable;
}
/**
 * Pihls uivatele.
 * Vytvoeno: (8.6.2004 10:39:03)
 * @return authentication.MyPrincipal je principal pro pihlenho uivatele. Jestlie pihlaovn sele, vrt <code>null</code>.
 * @param loginName java.lang.String je pihlaovac jmno.
 * @param loginPassword java.lang.String je pihlaovac heslo.
 * @exception java.sql.SQLException - problm s databz.
 */
public MyPrincipal login(String loginName, String loginPassword) throws java.sql.SQLException 
{
	java.sql.Statement statement = getConnection().createStatement();
	String userTable = getUserTable();
	String userRolesTable = getUserRoleTable();
	String userNameCol = getUserNameCol();
	String passwordCol = getUserCredCol();
	String roleNameCol = getRoleNameCol();
	String password = loginPassword;
	String digest = getDigest();
	if ((digest != null) && (!"".equals(digest)))
	{
	    password = Digest(password, digest);
	}
	String query = "SELECT " + roleNameCol + ", " + userTable + "." + userNameCol + " FROM " +
			userTable + ", " + userRolesTable + " WHERE " + userTable + "." +
			userNameCol + " = " + userRolesTable + "." + userNameCol + " AND " + 
			userTable + "." + userNameCol + " = '" + transform(loginName) + "' AND " + 
			passwordCol + " = '" + transform(password) + "'";
	java.sql.ResultSet rs = statement.executeQuery(query);
	MyPrincipal principal = null;
	while (rs.next())
	{
		if (principal == null)
		{
			principal = new MyPrincipal();
			principal.setName(rs.getString(userNameCol));
		}
		principal.addRole(rs.getString(roleNameCol));
	}
	rs.close();
	statement.close();
	return principal;
}
/**
 * Oteve nov spojen s databz.
 * @exception java.sql.SQLException - problm s databz.
 * @exception java.lang.ClassNotFoundException - problm s ovladaem.
 */
public void openConnection() throws java.sql.SQLException, ClassNotFoundException
{
	closeConnection();
	Class.forName(getDriverName());
	setConnection(java.sql.DriverManager.getConnection(getConnectionURL(), getConnectionName(), getConnectionPassword()));	
}
/**
 * Zaregistruje uivatele.
 * @param userName java.lang.String je jmno uivatele.
 * @param userPassword java.lang.String je heslo uivatele.
 * @param roles java.util.Set jsou role uivatele. 
 * @exception java.sql.SQLException - pote s databz.
 */
public void register(String userName, String userPassword, java.util.Vector roles) throws java.sql.SQLException 
{
	boolean autoCommit;
	java.sql.Connection con = getConnection();
	autoCommit = con.getAutoCommit();
	userName = transform(userName);
	userPassword = transform(userPassword);
	con.setAutoCommit(false);
	String userTable = getUserTable();
	String userRolesTable = getUserRoleTable();
	String userNameCol = getUserNameCol();
	String passwordCol = getUserCredCol();
	String roleNameCol = getRoleNameCol();
	String password = userPassword;
	String digest = getDigest();
	if ((digest != null) && (!"".equals(digest)))
	{
	    password = Digest(password, digest);
	}
	String insertUserQuery = "INSERT INTO " + userTable + "(" + userNameCol + ", " + passwordCol + ") VALUES ('" + userName + "', '" + transform(password) + "')";
	String insertRoleQuery = "INSERT INTO " + userRolesTable + "(" + roleNameCol + ", " + userNameCol + ") VALUES(?,'" + userName +"')";
	try
	{
		java.sql.Statement statement = con.createStatement();
		statement.executeUpdate(insertUserQuery);
		statement.close();
		java.sql.PreparedStatement pStatement = con.prepareStatement(insertRoleQuery);
		for(java.util.Enumeration e = roles.elements(); e.hasMoreElements();)
		{
			pStatement.setString(1, (String)e.nextElement());
			pStatement.executeUpdate();
		}
		con.commit();
		con.setAutoCommit(autoCommit);
	}
	catch (java.sql.SQLException ex)
	{
		con.rollback();
		con.setAutoCommit(autoCommit);
		throw ex;
	}
}
/**
 * Nastav spojen s databz.
 * @param newConnection java.sql.Connection je spojen s databz.
 */
public void setConnection(java.sql.Connection newConnection) 
{
	connection = newConnection;
}
/**
 * Nastav pihlaovac jmno do DB.
 * @param newConnectionName java.lang.String je pihlaovac jmno do DB.
 */
public void setConnectionName(java.lang.String newConnectionName) 
{
	if (newConnectionName == null)
	{
		newConnectionName = "";
	}
	connectionName = newConnectionName;
}
/**
 * Nastav pihlaovac heslo do DB.
 * @param newConnectionPassword java.lang.String je pihlaovac heslo do DB.
 */
public void setConnectionPassword(java.lang.String newConnectionPassword) 
{
	if (newConnectionPassword == null)
	{
		newConnectionPassword = "";
	}
	connectionPassword = newConnectionPassword;
}
/**
 * Nastav URL pro pipojen k DB.
 * @param newConnectionURL java.lang.String je URL pro pipojen k DB.
 */
public void setConnectionURL(String newConnectionURL) 
{
	if (newConnectionURL == null)
	{
		newConnectionURL = "";
	}
	connectionURL = newConnectionURL;
}
/**
 * Nastav ifrovac algoritmus pro ukldn hesel.
 * @param newDigest java.lang.String je ifrovac algoritmus pro ukldn hesel.
 */
public void setDigest(java.lang.String newDigest) 
{
	if (newDigest == null)
	{
		newDigest = "";
	}
	digest = newDigest;
}
/**
 * Nastav jmno JDBC ovladae pro prci s databz.
 * @param newDriverName java.lang.String je jmno JDBC ovladae pro prci s databz.
 */
public void setDriverName(java.lang.String newDriverName) 
{
	if (newDriverName == null)
	{
		newDriverName = "";
	}
	driverName = newDriverName;
}
/**
 * Nastav jzev sloupce se jmny rol.
 * @param newRoleNameCol java.lang.String je nzev sloupce se jmny rol.
 */
public void setRoleNameCol(java.lang.String newRoleNameCol)
{
	if (newRoleNameCol == null)
	{
		newRoleNameCol = "";
	}
	roleNameCol = newRoleNameCol;
}
/**
 * Nastav jmno sloupce s pstupovmi hesly.
 * @param newUserCredCol java.lang.String je jmno sloupce s pstupovmi hesly.
 */
public void setUserCredCol(java.lang.String newUserCredCol) 
{
	if (newUserCredCol == null)
	{
		newUserCredCol = "";
	}
	userCredCol = newUserCredCol;
}
/**
 * Nastav jmno sloupce s jmny uivatel.
 * @param newUserNameCol java.lang.String je jmno sloupce s jmny uivatel.
 */
public void setUserNameCol(java.lang.String newUserNameCol) 
{
	if (newUserNameCol == null)
	{
		newUserNameCol = "";
	}
	userNameCol = newUserNameCol;
}
/**
 * Nastav jmno tabulky rol.
 * @param newUserRoleTable java.lang.String je jmno tabulky rol.
 */
public void setUserRoleTable(java.lang.String newUserRoleTable) 
{
	if (newUserRoleTable == null)
	{
		newUserRoleTable = "";
	}
	userRoleTable = newUserRoleTable;
}
/**
 * Nastav jmno tabulky uivatel.
 * @param newUserTable java.lang.String je jmno tabulky uivatel.
 */
public void setUserTable(java.lang.String newUserTable) 
{
	if (newUserTable == null)
	{
		newUserTable = "";
	}
	userTable = newUserTable;
}
/**
 * Nahrad potenciln nebezpen znaky.
 * @return java.lang.String je etzec, kter bude soust SQL dotazu.
 * @param param java.lang.String je vstupn parametr s nahrazenmi nebezpenmi znaky.
 */
private String transform(String param) 
{
	try
	{
		param = param.replaceAll("\"", "\\\"");
		return param.replaceAll("'", "\'");
	}
	catch (java.util.regex.PatternSyntaxException ex)
	{
	}
	return "";
}
}
